/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.prognose;

import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import com.bitctrl.Constants;

import de.bsvrz.ibv.simbaustelle.simbaustelle.TestBaustelle;
import de.bsvrz.ibv.simbaustelle.simbaustelle.TestParameter;
import de.bsvrz.ibv.simbaustelle.simbaustelle.TestZeitRaum;
import de.bsvrz.ibv.simbaustelle.simbaustelle.TestZufluss;
import de.bsvrz.ibv.simbaustelle.verwaltung.BssStau;
import de.bsvrz.sys.funclib.bitctrl.util.UndefiniertException;

/**
 * Testklasse zur Überprüfung des Berechnungsalgorithmus für die Staubestimmung
 * einer Baustelle gemäß Testspezifikation.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class BaustellenSimulatorTest {

	/**
	 * Berechung der Stauprognose für eine Baustelle mit definierten
	 * Vorgabewerte und Vergleich mit den erwarteten Staus.
	 */
	@Test
	public final void prognoseBerechnen() {
		final BaustellenSimulator simulator = new BaustellenSimulator(new TestBaustelle(), null, null, null,
				new TestParameter());
		List<BssStau> stauListe = null;
		try {
			stauListe = simulator.prognoseErstellen(new TestZeitRaum(), new TestZufluss());
		} catch (final UndefiniertException e) {
			Assert.fail(e.getMessage());
		}

		Assert.assertNotNull("Es wurden keine Staus ermittelt", stauListe);
		Assert.assertEquals("Anzahl der erwarteten Staus ist nicht korrekt", 4, stauListe.size());

		BssStau stau = stauListe.get(0);
		Assert.assertEquals(6L * Constants.MILLIS_PER_HOUR, stau.getStartZeit());
		Assert.assertEquals(28800L * Constants.MILLIS_PER_SECOND, stau.getDauer());
		Assert.assertEquals(3006L, stau.getMaxLaenge());
		Assert.assertEquals(8L * Constants.MILLIS_PER_HOUR, stau.getMaxLaengeZeit());
		Assert.assertEquals(7229L, Math.round(stau.getVerlustZeit() / 1000.0));

		stau = stauListe.get(1);
		Assert.assertEquals(16L * Constants.MILLIS_PER_HOUR, stau.getStartZeit());
		Assert.assertEquals(21600L * Constants.MILLIS_PER_SECOND, stau.getDauer());
		Assert.assertEquals(2346L, stau.getMaxLaenge());
		Assert.assertEquals(18L * Constants.MILLIS_PER_HOUR, stau.getMaxLaengeZeit());
		Assert.assertEquals(6242L, Math.round(stau.getVerlustZeit() / 1000.0));

		stau = stauListe.get(2);
		Assert.assertEquals(30L * Constants.MILLIS_PER_HOUR, stau.getStartZeit());
		Assert.assertEquals(28800L * Constants.MILLIS_PER_SECOND, stau.getDauer());
		Assert.assertEquals(3462L, stau.getMaxLaenge());
		Assert.assertEquals(32L * Constants.MILLIS_PER_HOUR, stau.getMaxLaengeZeit());
		Assert.assertEquals(9212L, Math.round(stau.getVerlustZeit() / 1000.0));

		stau = stauListe.get(3);
		Assert.assertEquals(40L * Constants.MILLIS_PER_HOUR, stau.getStartZeit());
		Assert.assertEquals(21600L * Constants.MILLIS_PER_SECOND, stau.getDauer());
		Assert.assertEquals(2262L, stau.getMaxLaenge());
		Assert.assertEquals(42L * Constants.MILLIS_PER_HOUR, stau.getMaxLaengeZeit());
		Assert.assertEquals(6019L, Math.round(stau.getVerlustZeit() / 1000.0));
	}
}
