/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.simbaustelle;

import java.util.ArrayList;
import java.util.List;

import com.bitctrl.Constants;

import de.bsvrz.ibv.simbaustelle.prognose.BerechnungsZeitRaumSchnittstelle;

/**
 * Die Repräsentation des Testzeitraums für die Prognose der Staus einer
 * Baustelle.<br>
 * Die Standardinstanz der Klasse liefert den Berechnungszeitraum 0...48 Stunden
 * für die Prüfung des Berechnungsalgorithmus entsprechend Prüfspezifikation.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TestZeitRaum implements BerechnungsZeitRaumSchnittstelle {

	/**
	 * Prognose-Schrittweite für die Testausführung in Stunden.
	 */
	private static final int TEST_SCHRITTWEITE_IN_STUNDEN = 2;
	/**
	 * die Anzahl der Intervalle, die für den Testzeitraum geliefert werden
	 * sollen.
	 */
	private static final int ANZAHL_TEST_INTERVALLE = 24;

	/**
	 * Der Testzeitraum beginnt bei 0, d.h. für das Ende wird der Wert für die
	 * Anzahl der Intervalle multipliziert mit der Länge eines Prognoseschritts
	 * geliefert.
	 */
	@Override
	public final long getEndeZeitpunkt() {
		return TestZeitRaum.ANZAHL_TEST_INTERVALLE * TestZeitRaum.TEST_SCHRITTWEITE_IN_STUNDEN
				* Constants.MILLIS_PER_HOUR;
	}

	/**
	 * Der Testzeitraum ist laut Prüfspezifikation auf einen Zeitraum von Stunde
	 * 0 bis Stunde 48 Stunden gesetzt. Da das Prognoseintervall 2 Stunden
	 * beträgt, muss hier ein Feld mit Werten von 0 bis 44 als Stunden wert in
	 * Millisekunden geliefert werden.
	 */
	@Override
	public final List<Long> getIntervalle() {
		final List<Long> result = new ArrayList<>();
		long value = 0L;
		for (int idx = 0; idx < TestZeitRaum.ANZAHL_TEST_INTERVALLE; idx++) {
			result.add(value);
			value += TestZeitRaum.TEST_SCHRITTWEITE_IN_STUNDEN * Constants.MILLIS_PER_HOUR;
		}
		return result;
	}

	/**
	 * der Anfangszeitpunkt des Tests ist 0.
	 */
	@Override
	public final long getStartZeitpunkt() {
		return 0;
	}

	@Override
	public boolean isValid() {
		return true;
	}

	@Override
	public boolean prognoseMoeglich() {
		return true;
	}
}
